﻿/*
  Keyboard:
    Enter     - view image (also double click on item of image list)
    Ctrl+S    - save as
    Ctrl+W    - close view
    Alt+Enter - maximize/restore main window
    Esc       - quit
*/

#include "hmg.ch"

MEMVAR _HMG_SYSDATA

STATIC slKeepRatio := .T.
STATIC scTmpDir


FUNCTION Main()
  LOCAL aURL := { {"Eiffel Tower", "http://media.thisisinsider.com/images/58d919eaf2d0331b008b4bbd-960-720.jpg"}, ;
                  {"London Tower", "https://i2.wp.com/travelbluebook.com/wp-content/uploads/2016/02/Tower-of-London.jpg"}, ;
                  {"Pisa Tower",   "http://media1.britannica.com/eb-media/88/80588-004-0B5DCB41.jpg"}, ;
                  {"Taj Mahal",    "http://images.says.com/uploads/story_source/source_image/547108/2b63.jpg"}, ;
                  {"Tower Bridge", "http://www.goandco.co.uk/images/offices/tower-bridge.jpg"} }
  LOCAL n

  SET FONT TO "MS Shell Dlg", 8

  DEFINE WINDOW MainWnd;
    WIDTH  480 + GetSystemMetrics(7 /*SM_CXFIXEDFRAME*/) * 2;
    HEIGHT 320 + GetSystemMetrics(4 /*SM_CYCAPTION*/) + GetSystemMetrics(8 /*SM_CYFIXEDFRAME*/) * 2;
    TITLE  "Image from website";
    MAIN;
    ON PAINT    PaintFrames(MainWnd.HANDLE, 10, 170, MainWnd.CLIENTAREAWIDTH - 180, MainWnd.CLIENTAREAHEIGHT - 20);
    ON SIZE     MainWndResize();
    ON MAXIMIZE MainWndResize();
    ON RELEASE  HB_DirRemoveAll(scTmpDir)

    DEFINE LISTBOX PicLBox
      ROW         10
      COL         10
      WIDTH       150
      ONDBLCLICK  ImageView(aUrl)
      ONGOTFOCUS  PostMessage(MainWnd.ViewButton.HANDLE, 244 /*BM_SETSTYLE*/, 1 /*BS_DEFPUSHBUTTON*/, 1)
      ONLOSTFOCUS SendMessage(MainWnd.ViewButton.HANDLE, 244 /*BM_SETSTYLE*/, 0 /*BS_PUSHBUTTON*/, 1)
    END LISTBOX

    DEFINE CHECKBOX RatioCBox
      COL         10
      WIDTH       150
      HEIGHT      16
      VALUE       slKeepRatio
      CAPTION     "&Keep original aspect ratio"
      ONCHANGE    ((slKeepRatio := This.VALUE), MainWndResize())
      ONENTER     ImageView(aUrl)
      ONGOTFOCUS  PostMessage(MainWnd.ViewButton.HANDLE, 244 /*BM_SETSTYLE*/, 1 /*BS_DEFPUSHBUTTON*/, 1)
      ONLOSTFOCUS SendMessage(MainWnd.ViewButton.HANDLE, 244 /*BM_SETSTYLE*/, 0 /*BS_PUSHBUTTON*/, 1)
    END CHECKBOX

    DEFINE BUTTON ViewButton
      COL     10
      WIDTH   70
      HEIGHT  23
      CAPTION "View"
      ACTION  ImageView(aUrl)
    END BUTTON

    DEFINE BUTTON SaveButton
      COL     90
      WIDTH   70
      HEIGHT  23
      CAPTION "Save as"
      ACTION  ImageSaveAs(aUrl)
    END BUTTON

    DEFINE IMAGE PicImage
      STRETCH .T.
    END IMAGE

    ON KEY CONTROL+S  ACTION ImageSaveAs(aUrl)
    ON KEY CONTROL+W  ACTION ((MainWnd.PicImage.PICTURE := ""), MainWndResize())
    ON KEY ALT+RETURN ACTION If(MainWnd.IsMaximized, MainWnd.RESTORE, MainWnd.MAXIMIZE)
    ON KEY ESCAPE     ACTION MainWnd.RELEASE
  END WINDOW

  FOR n := 1 TO Len(aURL)
    MainWnd.PicLBox.AddItem(aURL[n][1])
  NEXT

  MainWnd.PicLBox.VALUE := 1

  EventCreate({ || SetMinMaxTrackSize(EventLPARAM(), 300, 170) }, MainWnd.HANDLE, 36 /*WM_GETMINMAXINFO*/)

  MainWndResize()
  MainWnd.CENTER
  MainWnd.ACTIVATE

RETURN NIL


FUNCTION MainWndResize()
  LOCAL nHBmp := MainWnd.PicImage.HBITMAP
  LOCAL nCAW  := MainWnd.CLIENTAREAWIDTH
  LOCAL nCAH  := MainWnd.CLIENTAREAHEIGHT
  LOCAL nPicR := 11
  LOCAL nPicC := 171
  LOCAL nPicW := nCAW - 182
  LOCAL nPicH := nCAH - 22
  LOCAL nPicRatio, nBmpRatio

  IF slKeepRatio .and. (nHBmp != 0)
    nPicRatio := nPicW / nPicH
    nBmpRatio := BT_BitmapWidth(nHBmp) / BT_BitmapHeight(nHBmp)

    IF nPicRatio < nBmpRatio
      nPicH := Int(nPicW / nBmpRatio)
      nPicR := Int((nCAH - nPicH) / 2)
    ELSEIF nPicRatio > nBmpRatio
      nPicW := Int(nPicH * nBmpRatio)
      nPicC += Int((nCAW - 182 - nPicW) / 2)
    ENDIF
  ENDIF

  MainWnd.PicLBox.HEIGHT   := nCAH - 79
  MainWnd.RatioCBox.ROW    := nCAH - 59
  MainWnd.ViewButton.ROW   := nCAH - 33
  MainWnd.SaveButton.ROW   := nCAH - 33
  MainWnd.PicImage.ROW     := nPicR
  MainWnd.PicImage.COL     := nPicC
  MainWnd.PicImage.WIDTH   := nPicW
  MainWnd.PicImage.HEIGHT  := nPicH

  MainWnd.PicImage.REDRAW

RETURN NIL


FUNCTION ImageView(aUrl)
  LOCAL nPos     := MainWnd.PicLBox.VALUE
  LOCAL cTmpFile := ImageDownload(aUrl, nPos)

  IF ! Empty(cTmpFile)
    MainWnd.PicImage.PICTURE := cTmpFile

    MainWndResize()
  ENDIF

RETURN NIL


FUNCTION ImageSaveAs(aUrl)
  LOCAL nPos := MainWnd.PicLBox.VALUE
  LOCAL cExt := "." + HB_TokenGet(aURL[nPos][2], HB_TokenCount(aURL[nPos][2], "."), ".")
  LOCAL cTargetFile
  LOCAL cTmpFile

  cTargetFile := PutFile({{"*" + cExt, "*" + cExt}}, NIL, HB_CWD(), .F., aURL[nPos][1], cExt)

  IF ! Empty(cTargetFile)
    cTmpFile := ImageDownload(aUrl, nPos)

    IF ! Empty(cTmpFile)
      IF HB_fCopy(cTmpFile, cTargetFile) != 0
        MsgBox("Can not save file:" + CRLF + cTargetFile)
      ENDIF
    ENDIF
  ENDIF

RETURN NIL


FUNCTION ImageDownload(aUrl, nPos)
  LOCAL cExt := "." + HB_TokenGet(aURL[nPos][2], HB_TokenCount(aURL[nPos][2], "."), ".")
  LOCAL cTmpFile

  IF Empty(scTmpDir) .or. (! HB_DirExists(scTmpDir))
    scTmpDir := DirTmpCreate()
  ENDIF

  cTmpFile := scTmpDir + HB_NtoS(nPos) + cExt

  IF ! HB_FileExists(cTmpFile)
    IF ! URLDownloadToFile(aURL[nPos][2], cTmpFile)
      cTmpFile := ""
      MsgBox("Can not download image:" + CRLF + aURL[nPos][2])
    ENDIF
  ENDIF

RETURN cTmpFile


FUNCTION DirTmpCreate()
  LOCAL cTmpDir := GetTempDir()
  LOCAL cName   := HB_fNameName(GetProgramFileName())
  LOCAL nCount  := 0

  HB_DirBuild(cTmpDir)

  DO WHILE HB_fNameExists(cTmpDir + cName + "_" + HB_NtoS(nCount))
    ++nCount
  ENDDO

  HB_DirCreate(cTmpDir += cName + "_" + HB_NtoS(nCount) + "\")

RETURN cTmpDir


#pragma BEGINDUMP

#include "SET_COMPILE_HMG_UNICODE.ch"
#include "HMG_UNICODE.h"

#include <Windows.h>
#include <urlmon.h>
#include "hbapi.h"


      // SetMinMaxTrackSize(lParam, nMinX, nMinY, nMaxX, nMaxY)
HB_FUNC( SETMINMAXTRACKSIZE )
{
  MINMAXINFO *MinMax = (MINMAXINFO *) HMG_parnl(1);

  if (hb_parni(2) > 0)
    MinMax->ptMinTrackSize.x = hb_parni(2);
  if (hb_parni(3) > 0)
    MinMax->ptMinTrackSize.y = hb_parni(3);
  if (hb_parni(4) > 0)
    MinMax->ptMaxTrackSize.x = hb_parni(4);
  if (hb_parni(5) > 0)
    MinMax->ptMaxTrackSize.y = hb_parni(5);
}


      // PaintFrames(nHWnd, nRow, nCol, nWidth, nHeight)
HB_FUNC( PAINTFRAMES )
{
  HWND        hWnd;
  PAINTSTRUCT ps;
  RECT        rc;
  HDC         hdc;

  hWnd = (HWND) HMG_parnl(1);
  hdc  = BeginPaint(hWnd, &ps);

  if (hdc)
  {
    if (! IsZoomed(hWnd))
    {
      GetClientRect(hWnd, &rc);

      rc.left = rc.right  - GetSystemMetrics(SM_CXVSCROLL);
      rc.top  = rc.bottom - GetSystemMetrics(SM_CYVSCROLL);

      DrawFrameControl(hdc, &rc, DFC_SCROLL, DFCS_SCROLLSIZEGRIP);
    }

    rc.left   = (LONG) hb_parnl(3);
    rc.top    = (LONG) hb_parnl(2);
    rc.right  = rc.left + (LONG) hb_parnl(4);
    rc.bottom = rc.top  + (LONG) hb_parnl(5);

    DrawEdge(hdc, &rc, EDGE_BUMP, BF_FLAT|BF_RECT);

    EndPaint(hWnd, &ps);
  }
}


      // https://msdn.microsoft.com/en-us/library/ms775123(v=vs.85).aspx
      // URLDownloadToFile(cURL, cFile)
HB_FUNC( URLDOWNLOADTOFILE )
{
  HRESULT hr = URLDownloadToFile(NULL, HMG_parc(1), HMG_parc(2), 0, NULL);
  
  hb_retl(hr == S_OK);
}

#pragma ENDDUMP
